@@ -1,11 +1,16 @@
 {-# LANGUAGE CPP #-}
-#if defined(__GLASGOW_HASKELL__) && __GLASGOW_HASKELL__ >= 702
-{-# LANGUAGE Safe #-}
+
+#if __GLASGOW_HASKELL__ >= 702 && __GLASGOW_HASKELL <= 706 && defined(MIN_VERSION_comonad) && !(MIN_VERSION_comonad(3,0,3))
+{-# LANGUAGE Trustworthy #-}
+#endif
+
+#if __GLASGOW_HASKELL__ >= 711
+{-# LANGUAGE ConstrainedClassMethods #-}
 #endif
 -----------------------------------------------------------------------------
 -- |
 -- Module      :  Data.Functor.Alt
--- Copyright   :  (C) 2011 Edward Kmett,
+-- Copyright   :  (C) 2011-2015 Edward Kmett
 -- License     :  BSD-style (see the file LICENSE)
 --
 -- Maintainer  :  Edward Kmett <ekmett@gmail.com>
@@ -19,6 +24,8 @@ module Data.Functor.Alt
   ) where
 
 import Control.Applicative hiding (some, many)
+import Control.Applicative.Backwards
+import Control.Applicative.Lift
 import Control.Arrow
 import Control.Exception (catch, SomeException)
 import Control.Monad
@@ -36,7 +43,10 @@ import qualified Control.Monad.Trans.State.Lazy as Lazy
 import qualified Control.Monad.Trans.Writer.Lazy as Lazy
 import Data.Functor.Apply
 import Data.Functor.Bind
-import Data.Semigroup
+import Data.Functor.Compose
+import Data.Functor.Product
+import Data.Functor.Reverse
+import Data.Semigroup hiding (Product)
 import Data.List.NonEmpty (NonEmpty(..))
 import Prelude (($),Either(..),Maybe(..),const,IO,Ord,(++),(.),either)
 
@@ -58,7 +68,7 @@ infixl 3 <!>
 -- If extended to an 'Alternative' then '<!>' should equal '<|>'.
 --
 -- Ideally, an instance of 'Alt' also satisfies the \"left distributon\" law of
--- MonadPlus with respect to <.>:
+-- MonadPlus with respect to '<.>':
 --
 -- > <.> right-distributes over <!>: (a <!> b) <.> c = (a <.> c) <!> (b <.> c)
 --
@@ -79,7 +89,7 @@ infixl 3 <!>
 -- > (m <!> n) >>= f = (m >>= f) <!> (m >>= f)
 
 class Functor f => Alt f where
-  -- | @(<|>)@ without a required @empty@
+  -- | '<|>' without a required @empty@
   (<!>) :: f a -> f a -> f a
 
   some :: Applicative f => f a -> f [a]
@@ -97,7 +107,7 @@ instance Alt (Either a) where
   Left _ <!> b = b
   a      <!> _ = a
 
--- | This instance does not actually satisfy the (<.>) right distributive law
+-- | This instance does not actually satisfy the ('<.>') right distributive law
 -- It instead satisfies the "Left-Catch" law
 instance Alt IO where
   m <!> n = catch m (go n) where
@@ -184,3 +194,20 @@ instance Alt f => Alt (Strict.RWST r w s f) where
 
 instance Alt f => Alt (Lazy.RWST r w s f) where
   Lazy.RWST m <!> Lazy.RWST n = Lazy.RWST $ \r s -> m r s <!> n r s
+
+instance Alt f => Alt (Backwards f) where
+  Backwards a <!> Backwards b = Backwards (a <!> b)
+
+instance (Alt f, Functor g) => Alt (Compose f g) where
+  Compose a <!> Compose b = Compose (a <!> b)
+
+instance Alt f => Alt (Lift f) where
+  Pure a   <!> _       = Pure a
+  Other _  <!> Pure b  = Pure b
+  Other a  <!> Other b = Other (a <!> b)
+
+instance (Alt f, Alt g) => Alt (Product f g) where
+  Pair a1 b1 <!> Pair a2 b2 = Pair (a1 <!> a2) (b1 <!> b2)
+
+instance Alt f => Alt (Reverse f) where
+  Reverse a <!> Reverse b = Reverse (a <!> b)
