@@ -1,8 +1,6 @@
-{-# LANGUAGE CPP #-}
 -----------------------------------------------------------------------------
 -- |
--- Module      :  Data.Semigroup.Foldable
--- Copyright   :  (C) 2011 Edward Kmett
+-- Copyright   :  (C) 2011-2015 Edward Kmett
 -- License     :  BSD-style (see the file LICENSE)
 --
 -- Maintainer  :  Edward Kmett <ekmett@gmail.com>
@@ -18,63 +16,19 @@ module Data.Semigroup.Foldable
   , for1_
   , sequenceA1_
   , foldMapDefault1
+  , asum1
   ) where
 
-import Control.Monad.Trans.Identity
 import Data.Foldable
-import Data.Functor.Identity
+import Data.Functor.Alt (Alt(..))
 import Data.Functor.Apply
-import Data.Functor.Product
-import Data.Functor.Compose
-import Data.List.NonEmpty (NonEmpty(..))
 import Data.Traversable.Instances ()
-import Data.Semigroup hiding (Product)
+import Data.Semigroup hiding (Product, Sum)
+import Data.Semigroup.Foldable.Class
 import Prelude hiding (foldr)
 
-#ifdef MIN_VERSION_containers
-import Data.Tree
-#endif
-
-#ifdef MIN_VERSION_comonad
-import Data.Functor.Coproduct
-#endif
-
-class Foldable t => Foldable1 t where
-  fold1 :: Semigroup m => t m -> m
-  foldMap1 :: Semigroup m => (a -> m) -> t a -> m
-
-  foldMap1 f = maybe (error "foldMap1") id . getOption . foldMap (Option . Just . f)
-  fold1 = foldMap1 id
-
-#ifdef MIN_VERSION_containers
-instance Foldable1 Tree where
-  foldMap1 f (Node a []) = f a
-  foldMap1 f (Node a (x:xs)) = f a <> foldMap1 (foldMap1 f) (x :| xs)
-#endif
-
-instance Foldable1 Identity where
-  foldMap1 f = f . runIdentity
-
-instance Foldable1 m => Foldable1 (IdentityT m) where
-  foldMap1 f = foldMap1 f . runIdentityT
-
-instance (Foldable1 f, Foldable1 g) => Foldable1 (Compose f g) where
-  foldMap1 f = foldMap1 (foldMap1 f) . getCompose
-
-instance (Foldable1 f, Foldable1 g) => Foldable1 (Product f g) where
-  foldMap1 f (Pair a b) = foldMap1 f a <> foldMap1 f b
-
-#ifdef MIN_VERSION_comonad
-instance (Foldable1 f, Foldable1 g) => Foldable1 (Coproduct f g) where
-  foldMap1 f = coproduct (foldMap1 f) (foldMap1 f)
-#endif
-
-instance Foldable1 NonEmpty where
-  foldMap1 f (a :| []) = f a
-  foldMap1 f (a :| b : bs) = f a <> foldMap1 f (b :| bs)
-
-instance Foldable1 ((,) a) where
-  foldMap1 f (_, x) = f x
+-- $setup
+-- >>> import Data.List.NonEmpty
 
 newtype JoinWith a = JoinWith {joinee :: (a -> a)}
 
@@ -136,3 +90,12 @@ foldMapDefault1 f = unwrapMonoid . foldMap (WrapMonoid . f)
 -- toStream :: Foldable1 t => t a -> Stream a
 -- concat1 :: Foldable1 t => t (Stream a) -> Stream a
 -- concatMap1 :: Foldable1 t => (a -> Stream b) -> t a -> Stream b
+
+newtype Alt_ f a = Alt_ { getAlt_ :: f a }
+
+instance Alt f => Semigroup (Alt_ f a) where
+  Alt_ a <> Alt_ b = Alt_ (a <!> b)
+
+asum1 :: (Foldable1 t, Alt m) => t (m a) -> m a
+asum1 = getAlt_ . foldMap1 Alt_
+{-# INLINE asum1 #-}
